#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_form.search module

This module provides a few helpers which can be used in search forms.
"""

from pyramid.response import Response
from z3c.form import button, field
from z3c.table.interfaces import IValues
from zope.interface import Interface, implementer
from zope.schema import TextLine

from pyams_form.form import AddForm
from pyams_form.interfaces.form import ISearchForm, IWidgetForm
from pyams_form.schema import ResetButton
from pyams_skin.interfaces import IContentSearch, IInnerPage, ISearchPage
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.table import BaseTable
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION


__docformat__ = 'restructuredtext'

from pyams_form import _  # pylint: disable=ungrouped-imports


class ISearchFields(Interface):
    """Default search form interface"""

    query = TextLine(title=_("Search query"),
                     required=False)


class ISearchButtons(Interface):
    """Search form buttons interface"""

    reset = ResetButton(name='reset', title=_("Reset"))
    search = button.Button(name='search', title=_("Search"))


@implementer(IWidgetForm, ISearchForm)
class SearchForm(AddForm):
    # pylint: disable=abstract-method
    """Base search form"""

    legend = _("Search")
    widget_icon_class = 'fa fa-fw fa-search'

    fields = field.Fields(ISearchFields)
    buttons = button.Buttons(ISearchButtons)

    ajax_handler = 'search-results.html'
    form_target = '#search-results'
    edit_permission = VIEW_SYSTEM_PERMISSION

    def updateActions(self):
        super(SearchForm, self).updateActions()
        if 'search' in self.actions:
            self.actions['search'].addClass('btn-primary')

    def get_search_results(self):
        """Get search results via an
        :py:class:`IContentSearch <pyams_skin.interfaces.IContentSearch>` adapter
        """
        registry = self.request.registry
        search = registry.queryMultiAdapter((self.context, self.request, self), IContentSearch)
        if search is None:
            search = registry.queryMultiAdapter((self.context, self.request), IContentSearch)
        if search is None:
            search = IContentSearch(self.context, None)
        if search is not None:
            self.updateWidgets()
            data, errors = self.extractData()  # pylint: disable=unused-variable
            return search.get_search_results(data)
        return None


@template_config(template='templates/search.pt', layer=IPyAMSLayer)
@implementer(IInnerPage, ISearchPage)
class SearchView:
    """Base search view"""

    search_form_factory = SearchForm
    search_form = None

    def update(self):
        """Update search view"""
        # pylint: disable=no-member
        self.search_form = self.search_form_factory(self.context, self.request)
        self.search_form.update()


class SearchResultsView(BaseTable):
    """Search results view"""

    search_form_factory = SearchForm

    def __call__(self):
        self.update()
        return Response(self.renderTable())


@adapter_config(context=(IContentSearch, IPyAMSLayer, SearchResultsView), provides=IValues)
class SearchResultsViewValuesAdapter(ContextRequestViewAdapter):
    """Search results view values adapter"""

    @property
    def values(self):
        """Get view search results"""
        form = self.view.search_form_factory(self.context, self.request)
        return form.get_search_results()
